@@ -1,4 +1,4 @@
-/*  $Id: xfcesmart.c 3315 2007-10-08 08:53:04Z afb $
+/*  $Id: xfcesmart.c 3719 2007-12-16 21:24:14Z afb $
  *
  *  xfce4-smartpm-plugin - a package manager applet for the xfce4 panel
  *  Copyright (c) 2007 Anders F Bjorklund <afb@users.sourceforge.net>
@@ -38,6 +38,10 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 
+#ifdef HAVE_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif
+
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #else
@@ -55,13 +59,13 @@ enum
   PACKAGE_MANAGER_SMART	= 0, /* use Smart Package Manager */
   PACKAGE_MANAGER_YUM	= 1, /* use Yum+Extender (Fedora) */
   PACKAGE_MANAGER_APT	= 2, /* use APT+Synaptic (Ubuntu) */
+  PACKAGE_MANAGER_SLAPT	= 3, /* use slapt+gslapt (Vector) */
 };
 
 #define SMART_WEBSITE "http://smartpm.org/"
 #define SMART_PROGRAM "smart" /* regular bin */
 #define SMART_UPDATE  "smart-update" /* suid */
 #define SMART_SUIDBIN "smart-root" /* Fedora */
-#define SMART_SUDOBIN "gksudo" /* Ubuntu/etc */
 
 #define YUM_WEBSITE   "http://www.yum-extender.org/"
 #define YUM_PROGRAM   "yumex"
@@ -71,6 +75,10 @@ enum
 #define APT_PROGRAM   "synaptic"
 #define APT_UPDATE    "apt-get"
 
+#define SLAPT_WEBSITE "http://software.jaos.org/#gslapt"
+#define SLAPT_PROGRAM /* "gslapt" */    PATH_GSLAPT
+#define SLAPT_UPDATE  /* "slapt-get" */ PATH_SLAPT_GET
+
 #define TIMEOUT_TIME  1000 /* milliseconds */
 
 #define PROGRESS_TIME 5*60 /* seconds */
@@ -96,9 +104,15 @@ smart_check_upgrades( SmartPlugin *smart);
 static void
 smart_set_manager(SmartPlugin *smart, gint manager);
 
+static void
+smart_show_notification(SmartPlugin *smart, gchar *summary, gchar *message);
+
 static void
 smart_sudo_smart_gui(SmartPlugin *smart, gchar *args);
 
+static void
+smart_launch_program(SmartPlugin *smart);
+
 static gboolean
 smart_size_changed (XfcePanelPlugin *plugin,
                      gint             size,
@@ -144,6 +158,8 @@ smart_set_available(SmartPlugin *smart, gint packages)
     gchar *s = g_strdup_printf((packages==1)?_("%d upgrade available"):
                                _("%d upgrades available"), packages);
     gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
+    if (packages != smart->updates_available)
+      smart_show_notification(smart, _("Package Upgrades"), s);
     g_free(s);
     if (smart->show_application)
       smart_set_icon(smart, smart->application_icon, smart->available_icon);
@@ -196,6 +212,9 @@ smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
         case PACKAGE_MANAGER_APT:
           /* TODO: determine number of new packages in APT channels */
           break;
+        case PACKAGE_MANAGER_SLAPT:
+          /* TODO: determine number of new packages in Slapt channels */
+          break;
       }
     }
     fclose(file);
@@ -233,6 +252,8 @@ smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
       s = g_strdup(_("Channels updated"));
 
     gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
+    if (packages != 0)
+      smart_show_notification(smart, _("Package Updates"), s);
     g_free(s);
   }
   else
@@ -310,6 +331,13 @@ smart_update_channels( SmartPlugin    *smart)
       argv[2] = "update";
       terminal = TRUE;
       break;
+    case PACKAGE_MANAGER_SLAPT:
+      argv[0] = PATH_SU;
+      argv[1] = "-c";
+      argv[2] = "\"" SLAPT_UPDATE " "\
+                "--update" "\"" ;
+      terminal = TRUE;
+      break;
     default:
       return 0;
   }
@@ -323,6 +351,7 @@ smart_update_channels( SmartPlugin    *smart)
   
     if (smart->package_manager == PACKAGE_MANAGER_YUM) title = YUM_UPDATE;
     if (smart->package_manager == PACKAGE_MANAGER_APT) title = APT_UPDATE;
+    if (smart->package_manager == PACKAGE_MANAGER_SLAPT) title = SLAPT_UPDATE;
   
     args = g_strdup_printf("%s %s %s",
                  argv[0], argv[1], argv[2]);
@@ -368,6 +397,7 @@ smart_update_channels( SmartPlugin    *smart)
     {
       /* smart-update not available */
       smart_sudo_smart_gui(smart, "update");
+      gtk_tooltips_set_tip(smart->tooltip, smart->button, _("Channels updated."), NULL);
     }
     else
     {
@@ -466,7 +496,13 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
               /* TODO: read package names */
             }
             break;
-        }
+           case PACKAGE_MANAGER_SLAPT:
+            /* 0 upgraded, 0 newly installed, 0 to remove and 0 not upgraded. */
+            if (strstr(p, " upgraded,") != NULL)
+              packages = atoi(p);
+            /* TODO: read package names */
+            break;
+       }
       }
       else
         seen_header = TRUE;
@@ -589,6 +625,11 @@ smart_check_upgrades( SmartPlugin    *smart)
       argv[7] = "--option"; /* only root is able to get a lock on the cache */
       argv[8] = "Debug::nolocking=true";
       break;
+    case PACKAGE_MANAGER_SLAPT:
+      argv[0] = SLAPT_UPDATE;
+      argv[1] = "--simulate";
+      argv[2] = (smart->dist_upgrade) ? "--dist-upgrade" : "--upgrade";
+      break;
     default:
       return 0;
   }
@@ -634,15 +675,7 @@ smart_set_manager(SmartPlugin *smart, gint manager)
 static gboolean
 smart_program_exists(gchar *program)
 {
-  gchar *path = "/usr/bin"; /* TODO: check PATH */
-  gchar *bin;
-  gboolean ok;
-  
-  bin = g_strdup_printf("%s/%s", path, program);
-  ok = g_file_test(bin, G_FILE_TEST_EXISTS);
-  g_free(bin);
-
-  return ok;
+  return g_file_test(program, G_FILE_TEST_EXISTS);
 }
 
 static void
@@ -655,25 +688,31 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
   switch (smart->package_manager)
   {
     case PACKAGE_MANAGER_SMART:
-      if (smart_program_exists(SMART_SUIDBIN))
-        program = g_strdup(SMART_SUIDBIN);
-      else /* if (smart_program_exists(SMART_SUDOBIN)) */
-        program = g_strdup_printf("%s -- %s", SMART_SUDOBIN, SMART_PROGRAM);
-      /*else
-        program = g_strdup(SMART_PROGRAM);*/
+      if (smart_program_exists("/usr/sbin/" SMART_SUIDBIN))
+        program = g_strdup("/usr/sbin/" SMART_SUIDBIN);
+      else if (smart_program_exists(PATH_GKSUDO))
+        program = g_strdup_printf("%s -- %s", PATH_GKSUDO, SMART_PROGRAM);
+      else
+        program = g_strdup(SMART_PROGRAM);
       break;
     case PACKAGE_MANAGER_YUM:
-      if (smart_program_exists(SMART_SUDOBIN))
-        program = g_strdup_printf("%s %s", SMART_SUDOBIN, YUM_PROGRAM);
+      if (smart_program_exists(PATH_GKSU))
+        program = g_strdup_printf("%s %s", PATH_GKSU, YUM_PROGRAM);
       else
         program = g_strdup(YUM_PROGRAM); /* it is using consolehelper */
       break;
     case PACKAGE_MANAGER_APT:
-      if (smart_program_exists(SMART_SUDOBIN))
-        program = g_strdup_printf("%s %s", SMART_SUDOBIN, APT_PROGRAM);
+      if (smart_program_exists(PATH_GKSUDO))
+        program = g_strdup_printf("%s %s", PATH_GKSUDO, APT_PROGRAM);
       else
         program = g_strdup(APT_PROGRAM); /* it is using consolehelper */
       break;
+    case PACKAGE_MANAGER_SLAPT:
+      if (smart_program_exists(PATH_GKSU))
+        program = g_strdup_printf("%s %s", PATH_GKSU, SLAPT_PROGRAM);
+      else
+        program = g_strdup(SLAPT_PROGRAM);
+      break;
     default:
       return;
   }
@@ -693,6 +732,88 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
   g_free(command);
 }
 
+static void
+smart_launch_program(SmartPlugin *smart)
+{
+  if (smart->upgrade_directly && smart->updates_available > 0)
+    smart_sudo_smart_gui(smart,
+      (smart->package_manager == PACKAGE_MANAGER_SLAPT) ? "--upgrade" : "upgrade");
+   else
+     smart_sudo_smart_gui(smart, NULL);
+}
+
+#ifdef HAVE_LIBNOTIFY
+static void
+smart_notify_callback(NotifyNotification *n, gchar *action, gpointer user_data)
+{
+  SmartPlugin *smart = user_data;
+  GError *error = NULL;
+
+  if (strcmp(action,"upgrade") == 0)
+    smart_launch_program(smart);
+
+  if (n != NULL)
+    notify_notification_close(n, &error);
+}
+#endif
+
+void
+smart_show_notification(SmartPlugin *smart, gchar *summary, gchar *message)
+{
+#ifdef HAVE_LIBNOTIFY
+  NotifyNotification *n;
+  GError *error = NULL;
+  XfceScreenPosition position;
+  GtkRequisition size;
+  int x,y;
+  
+  if (smart->show_notifications)
+  {
+    gdk_window_get_origin (GTK_WIDGET (smart->plugin)->window, &x, &y);
+    gtk_widget_size_request (GTK_WIDGET (smart->plugin), &size);
+ 
+    x = x + size.width/2;
+    y = y + size.height/2;
+
+    position = xfce_panel_plugin_get_screen_position (smart->plugin);
+
+    if (xfce_screen_position_is_horizontal(position))
+    {
+      if (xfce_screen_position_is_top(position)) y += size.height/2;
+      if (xfce_screen_position_is_bottom(position)) y -= size.height/2;
+    }
+    else /* xfce_screen_position_is_vertical */
+    {
+      if (xfce_screen_position_is_left(position)) x += size.width/2;
+      if (xfce_screen_position_is_right(position)) x -= size.width/2;
+    }
+
+    n = notify_notification_new (summary, message, NULL, NULL);
+    
+    notify_notification_add_action (n,
+      "default", "default",
+      smart_notify_callback, smart, NULL);
+    notify_notification_add_action (n,
+      "upgrade", _("Upgrade Now"),
+      smart_notify_callback, smart, NULL);
+
+    notify_notification_set_hint_int32 (n, "x", x);
+    notify_notification_set_hint_int32 (n, "y", y);
+
+    notify_notification_set_icon_from_pixbuf (n, smart->application_icon);
+    notify_notification_set_timeout (n, NOTIFY_EXPIRES_DEFAULT);
+    
+    if (!notify_notification_show (n, &error))
+    {
+      /*xfce_message_dialog(NULL, _("Notify Failed"), GTK_STOCK_DIALOG_ERROR,
+                          _("Failed to show notification"), error->message,
+                          GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);*/
+      g_error_free(error);
+    }
+  }
+#endif
+}
+
 void
 smart_save (XfcePanelPlugin *plugin,
              SmartPlugin    *smart)
@@ -721,6 +842,7 @@ smart_save (XfcePanelPlugin *plugin,
       xfce_rc_write_bool_entry (rc, "dist-upgrade", smart->dist_upgrade);
       xfce_rc_write_int_entry (rc, "manager", smart->package_manager);
       xfce_rc_write_bool_entry (rc, "appicon", smart->show_application);
+      xfce_rc_write_bool_entry (rc, "notify", smart->show_notifications);
 
       /* close the rc file */
       xfce_rc_close (rc);
@@ -754,6 +876,7 @@ smart_read (SmartPlugin *smart)
           smart->dist_upgrade =  xfce_rc_read_bool_entry (rc, "dist-upgrade", TRUE);
           smart->package_manager = xfce_rc_read_int_entry (rc, "manager", PACKAGE_MANAGER_SMART);
           smart->show_application =  xfce_rc_read_bool_entry (rc, "appicon", TRUE);
+          smart->show_notifications =  xfce_rc_read_bool_entry (rc, "notify", FALSE);
 
           /* cleanup */
           xfce_rc_close (rc);
@@ -771,6 +894,7 @@ smart_read (SmartPlugin *smart)
   smart->dist_upgrade = TRUE;
   smart->package_manager = PACKAGE_MANAGER_SMART;
   smart->show_application = TRUE;
+  smart->show_notifications = FALSE;
 }
 
 
@@ -800,10 +924,7 @@ smart_button_release_cb(GtkWidget *w, GdkEventButton *evt,
     switch(evt->button)
     {
       case 1:  /* left */
-        if (smart->upgrade_directly && smart->updates_available > 0)
-          smart_sudo_smart_gui(smart, "upgrade");
-        else
-          smart_sudo_smart_gui(smart, NULL);
+        smart_launch_program(smart);
         break;
       case 2:  /* middle */
         if (smart->in_progress)
@@ -859,7 +980,9 @@ smart_combobox_changed_cb(GtkComboBox *combo,
   button = g_object_get_data (G_OBJECT (smart->plugin), "dist-upgrade");
 
   if (button != NULL)
-  gtk_widget_set_sensitive (button, smart->package_manager == PACKAGE_MANAGER_APT);
+  gtk_widget_set_sensitive (button,
+     smart->package_manager == PACKAGE_MANAGER_APT ||
+     smart->package_manager == PACKAGE_MANAGER_SLAPT);
 
   if (smart->show_application) /* change the application icon too */
   smart_size_changed (smart->plugin, smart->current_icon_size, smart);
@@ -900,7 +1023,7 @@ smart_checkbox2_toggle_cb(GtkToggleButton *button,
 {
   SmartPlugin *smart = user_data;
 
-  smart->upgrade_directly = gtk_toggle_button_get_active(button);
+  smart->show_notifications = gtk_toggle_button_get_active(button);
 }
 
 static void
@@ -909,6 +1032,15 @@ smart_checkbox3_toggle_cb(GtkToggleButton *button,
 {
   SmartPlugin *smart = user_data;
 
+  smart->upgrade_directly = gtk_toggle_button_get_active(button);
+}
+
+static void
+smart_checkbox4_toggle_cb(GtkToggleButton *button,
+        gpointer user_data)
+{
+  SmartPlugin *smart = user_data;
+
   smart->dist_upgrade = gtk_toggle_button_get_active(button);
 }
 
@@ -975,6 +1107,10 @@ smart_new (XfcePanelPlugin *plugin)
 
   gtk_widget_destroy(dummy);
 
+#ifdef HAVE_LIBNOTIFY
+  notify_init("xfce4-smartpm-plugin");
+#endif
+
   /* set default icon and check */
   smart_size_changed (smart->plugin, smart->current_icon_size, smart);
   smart_set_available(smart, smart->updates_available);
@@ -996,6 +1132,10 @@ smart_free (XfcePanelPlugin *plugin,
   if (G_UNLIKELY (dialog != NULL))
     gtk_widget_destroy (dialog);
 
+#ifdef HAVE_LIBNOTIFY
+  notify_uninit();
+#endif
+
   /* destroy the panel widgets */
   gtk_widget_destroy (smart->button);
   gtk_widget_destroy (smart->menuitem);
@@ -1059,6 +1199,9 @@ smart_size_changed (XfcePanelPlugin *plugin,
     case PACKAGE_MANAGER_APT:
       iconname = "xfce4-" APT_PROGRAM;
       break;
+    case PACKAGE_MANAGER_SLAPT:
+      iconname = "xfce4-" SLAPT_PROGRAM;
+      break;
     default:
       iconname = "xfce-unknown";
       break;
@@ -1151,6 +1294,9 @@ smart_configure_response (GtkWidget    *dialog,
     case PACKAGE_MANAGER_APT:
       website = APT_WEBSITE;
       break;
+    case PACKAGE_MANAGER_SLAPT:
+      website = SLAPT_WEBSITE;
+      break;
     default:
       website = "http://goodies.xfce.org/";
       break;
@@ -1166,7 +1312,7 @@ smart_configure_response (GtkWidget    *dialog,
       g_free(s);
 
       if (G_UNLIKELY (result == FALSE))
-        g_warning (_("Unable to open the following url: %s"), SMART_WEBSITE);
+        g_warning (_("Unable to open the following url: %s"), website);
     }
   else if (response == GTK_RESPONSE_ACCEPT)
     {
@@ -1290,6 +1436,7 @@ smart_configure (XfcePanelPlugin *plugin,
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("Smart"));
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("Yum+Extender"));
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("APT+Synaptic"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("slapt+gslapt"));
   gtk_combo_box_set_active (GTK_COMBO_BOX(combo), smart->package_manager);
   gtk_widget_show (combo);
   g_signal_connect (G_OBJECT(combo),"changed", G_CALLBACK(smart_combobox_changed_cb), smart);
@@ -1317,20 +1464,31 @@ smart_configure (XfcePanelPlugin *plugin,
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);
 
+  button = gtk_check_button_new_with_label(_("Show notifications for updates and upgrades"));
+  gtk_widget_show (button);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->show_notifications);
+  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox2_toggle_cb), smart);
+  gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
+#ifndef HAVE_LIBNOTIFY
+  gtk_widget_set_sensitive (button, FALSE);
+#endif
+  
   button = gtk_check_button_new_with_label(_("Upgrade on launch (if upgrades available)"));
   gtk_widget_show (button);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->upgrade_directly);
-  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox2_toggle_cb), smart);
+  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox3_toggle_cb), smart);
   gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
 
   button = gtk_check_button_new_with_label(_("Smart upgrade to newer release (dist-upgrade)"));
   gtk_widget_show (button);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->dist_upgrade);
-  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox3_toggle_cb), smart);
+  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox4_toggle_cb), smart);
   gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
 
   g_object_set_data (G_OBJECT (plugin), "dist-upgrade", button);
-  gtk_widget_set_sensitive (button, smart->package_manager == PACKAGE_MANAGER_APT);
+  gtk_widget_set_sensitive (button,
+     smart->package_manager == PACKAGE_MANAGER_APT ||
+     smart->package_manager == PACKAGE_MANAGER_SLAPT);
 
   /* center dialog on the screen */
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
@@ -1363,7 +1521,7 @@ smart_about (XfcePanelPlugin *plugin)
               _("Check for updates using the Smart Package Manager"),
               XFCE_COPYRIGHT_TEXT("2007", "Anders F Bjorklund"),
               XFCE_LICENSE_GPL);
-  xfce_about_info_set_homepage(info, SMART_WEBSITE);
+  xfce_about_info_set_homepage(info, "http://goodies.xfce.org/projects/panel-plugins/xfce4-smartpm-plugin");
 
   icon = xfce_themed_icon_load("xfce4-smart", 48);
 
